import { Component, EventEmitter, Input, OnChanges, OnInit, Output, SimpleChanges } from '@angular/core';
import { FormBuilder, FormGroup } from '@angular/forms';
import { ActivatedRoute, Router } from '@angular/router';
import { ButtonType } from '../../shared/components/button/buttonType.enum';
import { Filter } from '../../structure-list/models/filter.model';
import { SearchService } from '../../structure-list/services/search.service';
import { TypeModal } from '../enums/TypeModal.enum';

@Component({
  selector: 'app-search-bar',
  templateUrl: './search-bar.component.html',
  styleUrls: ['./search-bar.component.scss'],
})
export class SearchBarComponent implements OnInit, OnChanges {
  @Input() shouldResetFilters: number = 0;
  @Input() shouldShowMore: number = 0;
  @Output() searchEvent = new EventEmitter();

  public locate = false;
  public addStructureFormModal = false;
  public buttonTypeEnum = ButtonType;
  public searchForm: FormGroup;
  public modalTypeOpened: TypeModal;
  public checkedFilterList: string[] = [];
  public numberEmployersChecked = 0;
  public numberJobChecked = 0;
  public jobTypes: string[] = [];
  public employerTypes: string[] = [];
  public queryString: string;

  public jobFilterChecked: string[] = [];
  public employerFilterChecked: string[] = [];

  constructor(
    private fb: FormBuilder,
    private activatedRoute: ActivatedRoute,
    private route: ActivatedRoute,
    private router: Router,
    private searchService: SearchService
  ) {
    this.searchForm = this.fb.group({
      searchTerm: this.activatedRoute.snapshot.queryParamMap.get('search')
        ? this.activatedRoute.snapshot.queryParamMap.get('search')
        : '',
    });
  }
  ngOnInit(): void {
    // Will store the different categories
    this.getData();
    this.queryString = this.activatedRoute.snapshot.queryParamMap.get('search');
    if (this.queryString) {
      const filters: Filter[] = [];
      filters.push(new Filter('query', this.queryString));
      this.searchEvent.emit(filters);
    }
  }
  ngOnChanges(changes: SimpleChanges): void {
    if (changes.shouldResetFilters && changes.shouldResetFilters.currentValue !== 0) this.resetFilters();
    if (changes.shouldShowMore && !changes.shouldShowMore.firstChange) {
      this.searchEvent.emit({
        queryParam: this.searchForm.get('searchTerm').value,
        page: changes.shouldShowMore.currentValue,
        jobFilters: this.jobFilterChecked,
        employerFilter: this.employerFilterChecked,
      });
    }
  }
  // Get the categories for each modal type
  private getData(): void {
    this.searchService.getJobs().subscribe((res) => {
      this.jobTypes = res;
    });
    this.searchService.getEmployers().subscribe((res) => {
      this.employerTypes = res;
    });
  }

  // Accessor to template angular.
  public get TypeModal(): typeof TypeModal {
    return TypeModal;
  }

  // Clear input search
  public clearInput(): void {
    this.searchForm.reset();
    this.applyFilter('');
  }

  public splitFilters(checkedFilterList: string[]): void {
    this.employerFilterChecked = checkedFilterList.filter((filter) => this.employerTypes.includes(filter));
    this.jobFilterChecked = checkedFilterList.filter((filter) => this.jobTypes.includes(filter));
  }
  // Sends an array containing all filters
  public applyFilter(term: string): void {
    this.shouldResetFilters = 0;
    // Add search input filter
    if (term) {
      this.router.navigate(['/annuaire'], {
        relativeTo: this.route,
        queryParams: {
          search: term,
        },
        queryParamsHandling: 'merge',
      });
    } else if (!term) {
      this.router.navigate(['/annuaire'], {
        relativeTo: this.route,
      });
    }
    this.splitFilters(this.checkedFilterList);
    this.searchEvent.emit({
      queryParam: term || '',
      jobFilters: this.jobFilterChecked,
      employerFilter: this.employerFilterChecked,
    });
  }

  public fetchResults(checkedFilters: string[]): void {
    this.checkedFilterList = checkedFilters;
    const inputTerm = this.searchForm.get('searchTerm').value;
    this.closeModal();
    this.applyFilter(inputTerm);
  }

  // Check if some modules is checked on filter and store number of modules checked
  public countCheckedFilters(): void {
    this.numberJobChecked = this.checkedFilterList.filter((filter) => this.jobTypes.includes(filter)).length;
    this.numberEmployersChecked = this.checkedFilterList.filter((filter) => this.employerTypes.includes(filter)).length;
  }

  public getModalCategory(): string[] | Error {
    switch (this.modalTypeOpened) {
      case TypeModal.jobs:
        return this.jobTypes;
      case TypeModal.employers:
        return this.employerTypes;
      default:
        throw new Error('Modal type not handle');
    }
  }

  // Open the modal and display the list according to the right filter button
  public openModal(modalType: TypeModal): void {
    // if modal already opened, reset type
    if (this.modalTypeOpened === modalType) {
      this.closeModal();
    } else if (this.modalTypeOpened !== modalType) {
      this.modalTypeOpened = modalType;
    }
  }

  public closeModal(): void {
    this.modalTypeOpened = undefined;
    this.countCheckedFilters();
  }

  public resetFilters(): void {
    this.checkedFilterList = [];
    this.numberEmployersChecked = 0;
    this.numberJobChecked = 0;
    this.employerFilterChecked = [];
    this.jobFilterChecked = [];
    this.searchEvent.emit({ queryParam: '', jobFilters: [], employerFilter: [] });
    this.router.navigate(['/annuaire']);
    this.searchForm.reset();
  }
  public removeFilter(filter: string): void {
    const index = this.checkedFilterList.findIndex((checkedFilter: string) => checkedFilter === filter);
    this.checkedFilterList.splice(index, 1);
    const inputTerm = this.searchForm.get('searchTerm').value;
    this.splitFilters(this.checkedFilterList);
    this.countCheckedFilters();
    this.searchEvent.emit({
      queryParam: inputTerm || '',
      jobFilters: this.checkedFilterList.length ? this.jobFilterChecked : [],
      employerFilter: this.checkedFilterList.length ? this.employerFilterChecked : [],
    });
  }
}
